'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var moment = _interopDefault(require('moment'));

var KeyValueEntry = /** @class */ (function () {
    function KeyValueEntry(type) {
        this._value = undefined;
        this._type = type;
    }
    Object.defineProperty(KeyValueEntry.prototype, "type", {
        get: function () {
            return this._type;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(KeyValueEntry.prototype, "id", {
        get: function () {
            return this._type.id;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(KeyValueEntry.prototype, "label", {
        get: function () {
            return this._type.label;
        },
        enumerable: true,
        configurable: true
    });
    KeyValueEntry.prototype.getSuggestions = function (prefix, entries, defaultEntries, allowDuplicates) {
        return this._type.getSuggestions(prefix, entries, defaultEntries, allowDuplicates);
    };
    KeyValueEntry.prototype.parse = function (val) {
        if (typeof val === "string") {
            return this._type.parseString(val);
        }
        else if (val !== undefined) {
            return this._type.checkValue(val);
        }
        return undefined;
    };
    KeyValueEntry.prototype.set = function (val) {
        this._value = this.parse(val);
        return this._value !== undefined;
    };
    Object.defineProperty(KeyValueEntry.prototype, "value", {
        get: function () {
            return this._value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(KeyValueEntry.prototype, "isPartial", {
        get: function () {
            return this.value === undefined;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(KeyValueEntry.prototype, "display", {
        get: function () {
            if (this.value === undefined) {
                return undefined;
            }
            return this._type.display(this.value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(KeyValueEntry.prototype, "editText", {
        get: function () {
            if (this.value === undefined) {
                return undefined;
            }
            return this._type.editText(this.value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(KeyValueEntry.prototype, "constructionData", {
        get: function () {
            return {
                key: this.id,
                value: this.editText,
            };
        },
        enumerable: true,
        configurable: true
    });
    KeyValueEntry.prototype.equals = function (entry) {
        return this._type.isEqual(this, entry);
    };
    return KeyValueEntry;
}());

var __spreadArrays = (undefined && undefined.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
var KeyValueType = /** @class */ (function () {
    function KeyValueType(id, label, aliases) {
        if (aliases === void 0) { aliases = []; }
        this._id = id;
        this._label = label;
        this._aliases = __spreadArrays([id, label], aliases).map(function (a) { return a.toLowerCase(); });
    }
    Object.defineProperty(KeyValueType.prototype, "id", {
        get: function () {
            return this._id;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(KeyValueType.prototype, "label", {
        get: function () {
            return this._label;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(KeyValueType.prototype, "aliases", {
        get: function () {
            return this._aliases;
        },
        enumerable: true,
        configurable: true
    });
    KeyValueType.prototype.findAlias = function (substring) {
        var substringLower = substring.toLowerCase();
        return this.aliases.find(function (a) { return a.includes(substringLower); });
    };
    KeyValueType.prototype.createPartialEntry = function () {
        return new KeyValueEntry(this);
    };
    KeyValueType.prototype.tryCreatePartialEntry = function (key) {
        if (!this.aliases.includes(key.toLowerCase())) {
            return undefined;
        }
        return new KeyValueEntry(this);
    };
    KeyValueType.prototype.tryCreateFullEntry = function (val) {
        var entry = new KeyValueEntry(this);
        if (entry.set(val)) {
            return entry;
        }
        else {
            return undefined;
        }
    };
    KeyValueType.prototype.isEqual = function (entryA, entryB) {
        return entryA.type === entryB.type && this.isValueEqual(entryA.value, entryB.value);
    };
    KeyValueType.prototype.isValueEqual = function (valueA, valueB) {
        return valueA === valueB;
    };
    return KeyValueType;
}());

var __extends = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();

var trueValues = ["ja", "yes", "wahr", "true", "1", "✓"];
var falseValues = ["nein", "no", "falsch", "false", "0", "✗"];
var BooleanKeyValueType = /** @class */ (function (_super) {
    __extends(BooleanKeyValueType, _super);
    function BooleanKeyValueType() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    BooleanKeyValueType.validate = function (val) {
        if (typeof val === "boolean") {
            return val;
        }
        else {
            throw new Package.meteor.Meteor.Error("invalid-boolean", "'" + val + "' is not a valid boolean.");
        }
    };
    BooleanKeyValueType.prototype.getSuggestions = function (prefix) {
        var substringLower = prefix.toLowerCase();
        var trueValue = trueValues.find(function (val) { return val.includes(substringLower); });
        var falseValue = falseValues.find(function (val) { return val.includes(substringLower); });
        return [
            { display: this.display(true), match: trueValue, value: trueValues[0] },
            { display: this.display(false), match: falseValue, value: falseValues[0] },
        ].filter(function (o) { return o.match !== undefined; });
    };
    BooleanKeyValueType.prototype.parseString = function (val) {
        var value = val.toLowerCase();
        if (value === "" || trueValues.includes(value)) {
            return true;
        }
        else if (falseValues.includes(value)) {
            return false;
        }
        else {
            return undefined;
        }
    };
    BooleanKeyValueType.prototype.checkValue = function (val) {
        if (typeof val === "boolean") {
            return val;
        }
        else {
            return undefined;
        }
    };
    BooleanKeyValueType.prototype.display = function (value) {
        if (value) {
            return { text: "✓" };
        }
        else {
            return { text: "✗" };
        }
    };
    BooleanKeyValueType.prototype.editText = function (value) {
        if (value) {
            return trueValues[0];
        }
        else {
            return falseValues[0];
        }
    };
    return BooleanKeyValueType;
}(KeyValueType));

var compOperators = {
    "<": {
        symbol: "<",
        selector: "$lt",
    },
    ">": {
        symbol: ">",
        selector: "$gt",
    },
    "<=": {
        symbol: "≤",
        selector: "$lte",
    },
    ">=": {
        symbol: "≥",
        selector: "$gte",
    },
    "!=": {
        symbol: "≠",
        selector: "$ne",
    },
    "=": {
        symbol: "=",
        selector: "$eq",
    },
};
var compOpsAndSymbols = ["<", ">", "<=", ">=", "!=", "=", "≤", "≥", "≠"];
function compOpToSymbol(op) {
    var _a, _b;
    return _b = (_a = compOperators[op]) === null || _a === void 0 ? void 0 : _a.symbol, (_b !== null && _b !== void 0 ? _b : op);
}
function compOpToSelector(op) {
    return compOperators[op].selector;
}
var mathOperators = {
    "=": "=",
    "*": "*",
    "/": "÷",
    "+": "+",
    "-": "-",
};
var mathOpsAndSymbols = ["=", "*", "/", "+", "-", "÷"];
function mathOpToSymbol(op) {
    var _a;
    return _a = mathOperators[op], (_a !== null && _a !== void 0 ? _a : op);
}

var __extends$1 = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var DateKeyValueType = /** @class */ (function (_super) {
    __extends$1(DateKeyValueType, _super);
    function DateKeyValueType() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._dateFormat = "YYYY-MM-DD";
        _this._dateParseFormats = ["YYYY-MM-DD", "YY-MM-DD", "MM-DD"];
        return _this;
    }
    DateKeyValueType.mongoSelectorFor = function (value) {
        var _a;
        return _a = {}, _a[compOpToSelector(value.operator)] = value.date, _a;
    };
    DateKeyValueType.mongoDateExpressionFor = function (value, field) {
        var _a, _b, _c, _d;
        switch (value.operator) {
            case "<":
            case ">=":
                return _a = {}, _a[field] = (_b = {}, _b[compOpToSelector(value.operator)] = moment(value.date).startOf("day").toDate(), _b), _a;
            case ">":
            case "<=":
                return _c = {}, _c[field] = (_d = {}, _d[compOpToSelector(value.operator)] = moment(value.date).endOf("day").toDate(), _d), _c;
            case "=":
                return { $and: [
                        this.mongoDateExpressionFor({ date: value.date, operator: "<=" }, field),
                        this.mongoDateExpressionFor({ date: value.date, operator: ">=" }, field),
                    ] };
            case "!=":
                return { $or: [
                        this.mongoDateExpressionFor({ date: value.date, operator: "<" }, field),
                        this.mongoDateExpressionFor({ date: value.date, operator: ">" }, field),
                    ] };
        }
    };
    Object.defineProperty(DateKeyValueType.prototype, "dateFormat", {
        set: function (dateFormat) {
            this._dateFormat = dateFormat;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateKeyValueType.prototype, "dateParseFormats", {
        set: function (dateParseFormats) {
            this._dateParseFormats = dateParseFormats;
        },
        enumerable: true,
        configurable: true
    });
    DateKeyValueType.prototype.getSuggestions = function (prefix) {
        var _this = this;
        var parsed = this.parseString(prefix);
        // empty input or invalid input or fallback "="
        if (prefix === "" || !parsed || (parsed.operator === "=" && !prefix.startsWith("="))) {
            var date_1 = new Date();
            var operators = Object.keys(compOperators);
            if (prefix !== "") {
                date_1 = parsed ? parsed.date : date_1;
                var filteredOperators = compOpsAndSymbols.filter(function (op) { return op.startsWith(prefix); });
                operators = filteredOperators.length > 0 ? filteredOperators : operators;
            }
            return operators.map(function (operator) { return ({
                display: _this.display({ operator: operator, date: date_1 }),
                value: _this.editText({ operator: operator, date: date_1 }),
            }); });
        }
        else {
            return [];
        }
    };
    DateKeyValueType.prototype.parseString = function (val) {
        var _a;
        var value = val.trim();
        if (value === "") {
            value = "=" + new Date();
        }
        value = value.replace(/\s/g, ""); // remove spaces
        var match = value.replace("≥", ">=")
            .replace("≤", "<=")
            .replace("<>", "!=")
            .replace("≠", "!=")
            .replace("==", "=")
            .match(/^(<=|<|>=|>|!=|=)?(.+)/);
        if (match) {
            var parsedDate = moment.utc(match[2], this._dateParseFormats, true);
            if (parsedDate.isValid()) {
                return {
                    operator: (_a = match[1], (_a !== null && _a !== void 0 ? _a : "=")),
                    date: parsedDate.toDate(),
                };
            }
            else {
                return undefined;
            }
        }
        else {
            return undefined;
        }
    };
    DateKeyValueType.prototype.checkValue = function (val) {
        if (Object.keys(compOperators).includes(val.operator) && val.date instanceof Date && !isNaN(val.date)) {
            return val;
        }
        else {
            return undefined;
        }
    };
    DateKeyValueType.prototype.display = function (value) {
        return { text: compOpToSymbol(value.operator) + " " + moment(value.date).format(this._dateFormat) };
    };
    DateKeyValueType.prototype.editText = function (value) {
        return this.display(value).text;
    };
    return DateKeyValueType;
}(KeyValueType));

function escapeRegexChars(text) {
    return text.replace(/[.^$*+\-?()[\]{}\\|]/g, "\\$&");
}

var __extends$2 = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var NumberKeyValueType = /** @class */ (function (_super) {
    __extends$2(NumberKeyValueType, _super);
    function NumberKeyValueType() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._delimiters = {
            decimal: ".",
            groups: ",",
        };
        _this._groupsDelimiterRegex = /,/gi;
        return _this;
    }
    NumberKeyValueType.mongoSelectorFor = function (value, numberTransform) {
        var _a;
        if (numberTransform === void 0) { numberTransform = function (n) { return n; }; }
        return _a = {}, _a[compOpToSelector(value.operator)] = numberTransform(value.number), _a;
    };
    Object.defineProperty(NumberKeyValueType.prototype, "delimiters", {
        set: function (delimiters) {
            this._delimiters = delimiters;
            this._groupsDelimiterRegex = new RegExp(escapeRegexChars(this._delimiters.groups), "gi");
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NumberKeyValueType.prototype, "numberFormatter", {
        set: function (numberFormatter) {
            this._numberFormatter = numberFormatter;
        },
        enumerable: true,
        configurable: true
    });
    NumberKeyValueType.prototype.getSuggestions = function (prefix) {
        var _this = this;
        var parsed = this.parseString(prefix);
        // empty input or invalid input or fallback "="
        if (prefix === "" || !parsed || (parsed.operator === "=" && !prefix.startsWith("="))) {
            var number_1 = 42;
            var operators = Object.keys(compOperators);
            if (prefix !== "") {
                number_1 = parsed ? parsed.number : number_1;
                var filteredOperators = compOpsAndSymbols.filter(function (op) { return op.startsWith(prefix); });
                operators = filteredOperators.length > 0 ? filteredOperators : operators;
            }
            return operators.map(function (operator) { return ({
                display: _this.display({ operator: operator, number: number_1 }),
                value: _this.editText({ operator: operator, number: number_1 }),
            }); });
        }
        else {
            return [];
        }
    };
    NumberKeyValueType.prototype.parseString = function (val) {
        var _a;
        var value = val.trim();
        if (value === "") {
            value = ">0";
        }
        value = value.replace(/\s/g, "") // remove spaces
            .replace(this._groupsDelimiterRegex, "") // remove grouping delimiters
            .replace(this._delimiters.decimal, "."); // convert to decimal point
        var match = value.replace("≥", ">=")
            .replace("≤", "<=")
            .replace("<>", "!=")
            .replace("≠", "!=")
            .replace("==", "=")
            .match(/^(<=|<|>=|>|!=|=)?(-?\d*\.?\d+)/);
        if (match) {
            return {
                operator: (_a = match[1], (_a !== null && _a !== void 0 ? _a : "=")),
                number: Number(match[2]),
            };
        }
        else {
            return undefined;
        }
    };
    NumberKeyValueType.prototype.checkValue = function (val) {
        if (Object.keys(compOperators).includes(val.operator) && typeof val.number === "number") {
            return val;
        }
        else {
            return undefined;
        }
    };
    NumberKeyValueType.prototype.display = function (value) {
        var numberString = this._numberFormatter ? this._numberFormatter.format(value.number) : String(value.number);
        return { text: compOpToSymbol(value.operator) + " " + numberString };
    };
    NumberKeyValueType.prototype.editText = function (value) {
        return this.display(value).text;
    };
    NumberKeyValueType.prototype.isValueEqual = function (valueA, valueB) {
        if (_super.prototype.isValueEqual.call(this, valueA, valueB)) {
            return true;
        }
        if (valueA && valueB) {
            return valueA.operator === valueB.operator && valueA.number === valueB.number;
        }
        return false;
    };
    return NumberKeyValueType;
}(KeyValueType));

var __extends$3 = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var NumberModifierKeyValueType = /** @class */ (function (_super) {
    __extends$3(NumberModifierKeyValueType, _super);
    function NumberModifierKeyValueType() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._delimiters = {
            decimal: ".",
            groups: ",",
        };
        _this._groupsDelimiterRegex = /,/gi;
        return _this;
    }
    NumberModifierKeyValueType.apply = function (value, number, roundMode) {
        if (roundMode === void 0) { roundMode = "not"; }
        var result;
        switch (value.operator) {
            case "=":
                result = value.number;
                break;
            case "*":
                result = number * value.number;
                break;
            case "/":
                result = number / value.number;
                break;
            case "+":
                result = number + value.number;
                break;
            case "-":
                result = number - value.number;
                break;
            default:
                throw new Package.meteor.Meteor.Error("invalid-number-modifier", "'" + value.operator + "' is not a valid operation on numbers.");
        }
        switch (roundMode) {
            case "up":
                result = Math.ceil(result);
                break;
            case "down":
                result = Math.floor(result);
                break;
            case "math":
                result = Math.round(result);
                break;
            // no rounding
        }
        return result;
    };
    Object.defineProperty(NumberModifierKeyValueType.prototype, "delimiters", {
        set: function (delimiters) {
            this._delimiters = delimiters;
            this._groupsDelimiterRegex = new RegExp(escapeRegexChars(this._delimiters.groups), "gi");
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NumberModifierKeyValueType.prototype, "numberFormatter", {
        set: function (numberFormatter) {
            this._numberFormatter = numberFormatter;
        },
        enumerable: true,
        configurable: true
    });
    NumberModifierKeyValueType.prototype.getSuggestions = function (prefix) {
        var _this = this;
        var parsed = this.parseString(prefix);
        // empty input or invalid input or fallback "="
        if (prefix === "" || !parsed || (parsed.operator === "=" && !prefix.startsWith("="))) {
            var number_1 = 42;
            var operators = Object.keys(mathOperators);
            if (prefix !== "") {
                number_1 = parsed ? parsed.number : number_1;
                var filteredOperators = mathOpsAndSymbols.filter(function (op) { return op.startsWith(prefix); });
                operators = filteredOperators.length > 0 ? filteredOperators : operators;
            }
            return operators.map(function (operator) { return ({
                display: _this.display({ operator: operator, number: number_1 }),
                value: _this.editText({ operator: operator, number: number_1 }),
            }); });
        }
        else {
            return [];
        }
    };
    NumberModifierKeyValueType.prototype.parseString = function (val) {
        var _a;
        var value = val.trim();
        if (value === "") {
            return undefined;
        }
        value = value.replace(/\s/g, "") // remove spaces
            .replace(this._groupsDelimiterRegex, "") // remove grouping delimiters
            .replace(this._delimiters.decimal, "."); // convert to decimal point
        var match = value.replace("÷", "/")
            .match(/^([=*/+-])?(-?\d*\.?\d+)/);
        if (match) {
            return {
                operator: (_a = match[1], (_a !== null && _a !== void 0 ? _a : "=")),
                number: Number(match[2]),
            };
        }
        else {
            return undefined;
        }
    };
    NumberModifierKeyValueType.prototype.checkValue = function (val) {
        if (Object.keys(mathOperators).includes(val.operator) && typeof val.number === "number") {
            return val;
        }
        else {
            return undefined;
        }
    };
    NumberModifierKeyValueType.prototype.display = function (value) {
        var numberString = this._numberFormatter ? this._numberFormatter.format(value.number) : String(value.number);
        return { text: mathOpToSymbol(value.operator) + " " + numberString };
    };
    NumberModifierKeyValueType.prototype.editText = function (value) {
        return this.display(value).text;
    };
    NumberModifierKeyValueType.prototype.isValueEqual = function (valueA, valueB) {
        if (_super.prototype.isValueEqual.call(this, valueA, valueB)) {
            return true;
        }
        if (valueA && valueB) {
            return valueA.operator === valueB.operator && valueA.number === valueB.number;
        }
        return false;
    };
    return NumberModifierKeyValueType;
}(KeyValueType));

var __extends$4 = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var RegexKeyValueType = /** @class */ (function (_super) {
    __extends$4(RegexKeyValueType, _super);
    function RegexKeyValueType() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    RegexKeyValueType.mongoSelectorFor = function (value) {
        return { $regex: value.regex, $options: value.options };
    };
    RegexKeyValueType.prototype.getSuggestions = function (prefix) {
        var _this = this;
        var display = function (val) { return _this.display(_this.parseString(val)); };
        return [
            { display: display("abc"), value: "abc", extra: "Einfache Suche" },
            { display: display('"Hallo Welt!"'), value: '"Hallo Welt!"', extra: "\"...\" für Leerzeichen" },
            { display: display("/[a-c0-9]/i"), value: "/[a-c0-9]/i", extra: "RegEx" },
        ];
    };
    RegexKeyValueType.prototype.parseString = function (val) {
        var text = val.trim();
        if (text === "") {
            return undefined;
        }
        var match = text.match(/^\/(.*)\/([a-z]*)$/);
        if (match) {
            try {
                // tslint:disable-next-line:no-unused-expression as the expression is used for error checking
                new RegExp(match[1], match[2]);
                return {
                    text: text,
                    printRegex: jQuery("<div>").text(match[1]).html(),
                    regex: match[1],
                    options: match[2],
                };
            }
            catch (e) {
                // escape below like non-regex strings
            }
        }
        return {
            text: text,
            regex: escapeRegexChars(text),
            options: "i",
        };
    };
    RegexKeyValueType.prototype.checkValue = function (val) {
        if (typeof val.text === "string"
            && (val.printRegex === undefined || typeof val.printRegex === "string")
            && typeof val.regex === "string"
            && typeof val.options === "string") {
            return val;
        }
        else {
            return undefined;
        }
    };
    RegexKeyValueType.prototype.display = function (value) {
        if (value.printRegex) {
            return { html: "<code>/</code>" + value.printRegex + "<code>/" + value.options + "</code>" };
        }
        else {
            return { text: value.text };
        }
    };
    RegexKeyValueType.prototype.editText = function (value) {
        if (value.text.includes(" ")) {
            return "\"" + value.text.replace(/\\/g, "\\\\").replace(/"/g, "\\\"") + "\"";
        }
        else {
            return value.text;
        }
    };
    RegexKeyValueType.prototype.isValueEqual = function (valueA, valueB) {
        if (_super.prototype.isValueEqual.call(this, valueA, valueB)) {
            return true;
        }
        if (valueA && valueB) {
            return valueA.regex === valueB.regex;
        }
        return false;
    };
    return RegexKeyValueType;
}(KeyValueType));

var __extends$5 = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var ascendingValues = ["aufsteigend", "auf", "asc", "ascending", "up", "1", "+"];
var descendingValues = ["absteigend", "ab", "desc", "descending", "down", "-1", "-"];
var SortKeyValueType = /** @class */ (function (_super) {
    __extends$5(SortKeyValueType, _super);
    function SortKeyValueType() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    SortKeyValueType.mongoSpecifierFor = function (value) {
        return value ? "asc" : "desc";
    };
    SortKeyValueType.prototype.getSuggestions = function (prefix) {
        var substringLower = prefix.toLowerCase();
        var ascendingValue = ascendingValues.find(function (val) { return val.includes(substringLower); });
        var descendingValue = descendingValues.find(function (val) { return val.includes(substringLower); });
        return [
            { display: this.display(true), match: ascendingValue, value: ascendingValues[0] },
            { display: this.display(false), match: descendingValue, value: descendingValues[0] },
        ].filter(function (o) { return o.match !== undefined; });
    };
    SortKeyValueType.prototype.parseString = function (val) {
        var value = val.trim().toLowerCase();
        if (value === "" || ascendingValues.includes(value)) {
            return true;
        }
        else if (descendingValues.includes(value)) {
            return false;
        }
        else {
            return undefined;
        }
    };
    SortKeyValueType.prototype.checkValue = function (val) {
        if (typeof val === "boolean") {
            return val;
        }
        else {
            return undefined;
        }
    };
    SortKeyValueType.prototype.display = function (value) {
        if (value) {
            return { html: "<i class='fa fa-sort-up'></i>" };
        }
        else {
            return { html: "<i class='fa fa-sort-down'></i>" };
        }
    };
    SortKeyValueType.prototype.editText = function (value) {
        if (value) {
            return ascendingValues[0];
        }
        else {
            return descendingValues[0];
        }
    };
    return SortKeyValueType;
}(KeyValueType));

var __extends$6 = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
function stringToTag(value) {
    return { value: value, display: { text: value } };
}
function isTags(tags) {
    return tags.length === 0 || typeof tags[0] !== "string";
}
var TagKeyValueType = /** @class */ (function (_super) {
    __extends$6(TagKeyValueType, _super);
    function TagKeyValueType() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._tags = [];
        _this._allowOther = false;
        return _this;
    }
    Object.defineProperty(TagKeyValueType.prototype, "tags", {
        set: function (tags) {
            if (isTags(tags)) {
                this._tags = tags;
            }
            else {
                this._tags = tags.map(stringToTag);
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TagKeyValueType.prototype, "allowOther", {
        set: function (allowOther) {
            this._allowOther = allowOther;
        },
        enumerable: true,
        configurable: true
    });
    TagKeyValueType.prototype.getSuggestions = function (prefix, entries, defaultEntries, allowDuplicates) {
        var _this = this;
        var substringLower = prefix.toLowerCase();
        var tags = this._tags.filter(function (t) { var _a; return t.value.toLowerCase().includes(substringLower) || ((_a = t.editString) === null || _a === void 0 ? void 0 : _a.toLowerCase().includes(substringLower)); });
        if (!allowDuplicates) {
            tags = tags.filter(function (t) {
                return entries.every(function (e) { return e.type !== _this || !_this.isValueEqual(t, e.value); }) &&
                    defaultEntries.every(function (e) { return e.type !== _this || !_this.isValueEqual(t, e.value); });
            });
        }
        return tags.map(function (t) {
            var _a;
            return ({
                display: _this.display(t),
                match: prefix !== "" ?
                    ((_a = t.editString, (_a !== null && _a !== void 0 ? _a : "")).toLowerCase().includes(substringLower) ? t.editString : t.value)
                    : undefined,
                value: t.value,
            });
        });
    };
    TagKeyValueType.prototype.parseString = function (val) {
        var value = val.trim().toLowerCase();
        var tag = this._tags.find(function (t) { var _a; return t.value.toLowerCase() === value || ((_a = t.editString) === null || _a === void 0 ? void 0 : _a.toLowerCase()) === value; });
        if (tag) {
            return tag;
        }
        if (this._allowOther && val.trim() !== "") {
            return stringToTag(val.trim());
        }
    };
    TagKeyValueType.prototype.checkValue = function (val) {
        if (typeof val === "string") {
            return this.parseString(val);
        }
        else {
            return undefined;
        }
    };
    TagKeyValueType.prototype.display = function (value) {
        var _a, _b;
        return _a = value.display, (_a !== null && _a !== void 0 ? _a : { text: (_b = value.editString, (_b !== null && _b !== void 0 ? _b : value.value)) });
    };
    TagKeyValueType.prototype.editText = function (value) {
        var _a;
        return _a = value.editString, (_a !== null && _a !== void 0 ? _a : value.value);
    };
    TagKeyValueType.prototype.isValueEqual = function (valueA, valueB) {
        if (_super.prototype.isValueEqual.call(this, valueA, valueB)) {
            return true;
        }
        if (valueA && valueB) {
            return valueA.value === valueB.value;
        }
        return false;
    };
    return TagKeyValueType;
}(KeyValueType));

function createEntriesFromTypes(entries, types) {
    return entries.map(function (e) { return createEntry(e, types); })
        .filter(function (entry) { return entry !== undefined; });
}
function createEntry(e, types) {
    for (var _i = 0, types_1 = types; _i < types_1.length; _i++) {
        var type = types_1[_i];
        if (type.id === e.key) {
            return type.tryCreateFullEntry(e.value);
        }
    }
    return undefined;
}

function clearEntries(keyValueInput) {
    keyValueInput.dispatchEvent(new CustomEvent("clearEntries"));
}

exports.BooleanKeyValueType = BooleanKeyValueType;
exports.DateKeyValueType = DateKeyValueType;
exports.KeyValueEntry = KeyValueEntry;
exports.KeyValueType = KeyValueType;
exports.NumberKeyValueType = NumberKeyValueType;
exports.NumberModifierKeyValueType = NumberModifierKeyValueType;
exports.RegexKeyValueType = RegexKeyValueType;
exports.SortKeyValueType = SortKeyValueType;
exports.TagKeyValueType = TagKeyValueType;
exports.clearEntries = clearEntries;
exports.createEntriesFromTypes = createEntriesFromTypes;
